'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _Object$keys = _interopDefault(require('babel-runtime/core-js/object/keys'));
var _JSON$stringify = _interopDefault(require('babel-runtime/core-js/json/stringify'));
var axios = _interopDefault(require('axios'));
var bex = _interopDefault(require('bcorejs'));
var Mnemonic = _interopDefault(require('bitcore-mnemonic'));
var _extends = _interopDefault(require('babel-runtime/helpers/extends'));

/**
 * Get last blocks
 * @return {Promise<Response>} Last blocks array
 */
var getBlocks = function getBlocks(options) {
  var params = queryBuilder(options);
  return query('api/blocks?' + params).then(function (res) {
    return res.blocks;
  });
};

/**
 * Get block from blockId
 * @param {string} - Block id
 * @return {Promise<Response>} Block details
 */
var getBlock = function getBlock(blockId) {
  return query('api/blocks/get', {
    id: blockId
  }).then(function (res) {
    return res.block;
  });
};

/**
 * Get blockchain height
 * @return {Promise<Response>} Block height
 */
var getBlocksHeight = function getBlocksHeight() {
  return query('api/blocks/getHeight').then(function (res) {
    return res.height;
  });
};

/**
 * Get blockchain transaction fee
 * @return {Promise<Response>} Transaction fee
 */
var getBlockchainFee = function getBlockchainFee() {
  return query('api/blocks/getFee').then(function (res) {
    return res.fee;
  });
};

/**
 * Get net hash
 * @return {Promise<Response>} Nethash
 */
var getNetHash = function getNetHash() {
  return query('api/blocks/getNetHash').then(function (res) {
    return res.nethash;
  });
};

/**
 * Get blockchain's epoch
 * @return {Promise<Response>} Epoch
 */
var getEpoch = function getEpoch() {
  return query('api/blocks/getEpoch').then(function (res) {
    return res.epoch;
  });
};

/**
 * Get block forging reward
 * @return {Promise<Response>} Reward
 */
var getReward = function getReward() {
  return query('api/blocks/getReward').then(function (res) {
    return res.reward;
  });
};

/**
 * Get supply
 * @return {Promise<Response>} Supply
 */
var getSupply = function getSupply() {
  return query('api/blocks/getSupply').then(function (res) {
    return res.supply;
  });
};

/**
 * Get blockchain status
 * @return {Promise<Response>} Status object
 */
var getStatus = function getStatus() {
  return query('api/blocks/getStatus').then(function (res) {
    return res;
  });
};

var seeds = {
  BENCH: ['http://51.15.99.156:6620', 'http://51.15.113.19:6620', 'http://51.15.101.113:6620', 'http://51.15.121.39:6620', 'http://51.15.86.219:6620'],
  BENCHTEST: ['http://node1.bex.life']
};

var networksType = {
  BENCH: {
    label: 'Bench',
    version: 25,
    port: 6620
  },
  BENCHTEST: {
    label: 'BenchTest',
    version: 65,
    port: 6624
  }
};

var bcorejs = bex;
var currentNetwork = networksType.BENCH;
var endpoint = seeds.BENCH[0];

/**
 * Query API
 * @return {Promise<Response>} Query result
 */
var query = function query(url, params) {
  return axios.get(getEndpoint() + '/' + url, {
    params: params
  }).then(function (res) {
    return res.data;
  }).catch(function (err) {
    return err;
  });
};

/**
 * Post to API
 * @return {Promise<Response>} Post result
 */
var post = function post(url, data) {
  return getNetHash().then(function (nethash) {
    var dataReq = _JSON$stringify({ transactions: [data] });
    return axios.post(getEndpoint() + '/' + url, dataReq, {
      headers: {
        'Content-Type': 'application/json',
        'version': '0.3.0',
        'port': 1,
        'nethash': nethash
      }
    }).then(function (response) {
      return response.data;
    }).catch(function (err) {
      return err;
    });
  });
};

/**
 * Return API endpoint
 * @return {string} Endpoint
 */
var getEndpoint = function getEndpoint() {
  return endpoint;
};

/**
 * Get network
 * @return {Object} Current network
 */
var getNetwork = function getNetwork() {
  return currentNetwork;
};

/**
 * Set network
 * @param {string} network - Network name to use
 * @param {string} url - Set a custom API endpoint
 */
var setNetwork = function setNetwork(network) {
  var url = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

  if (network != null) network = formatNetworkType(network);
  switch (network) {
    case networksType.BENCHTEST.label:
      {
        currentNetwork = networksType.BENCHTEST;
        bcorejs.crypto.setNetworkVersion(networksType.BENCHTEST.version);
        endpoint = seeds.BENCHTEST[0];
      }break;
    case networksType.BENCH.label:
      {
        currentNetwork = networksType.BENCH;
        bcorejs.crypto.setNetworkVersion(networksType.BENCH.version);
        endpoint = seeds.BENCH[0];
      }break;
  }
  if (url != null) {
    if (!url.startsWith('http')) url = 'http://' + url;
    endpoint = url;
  }
};

/**
 * Get seeds for the current network type
 * If network given, return seeds of this network
 * @return {array} Seeds list
 */
var getSeeds = function getSeeds() {
  var network = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

  if (network != null) {
    network = formatNetworkType(network); // Format network label
  } else {
    return seeds[getNetwork().label.toUpperCase()];
  }
  switch (network) {
    case networksType.BENCH.label:
      return seeds.BENCH;
    case networksType.BENCHTEST.label:
      return seeds.BENCHTEST;
  }
};

/**
 * Get all seeds
 * @return {Object} Seeds list
 */
var getAllSeeds = function getAllSeeds() {
  return seeds;
};

/**
 * Build url query from object
 * @param {Object} - Query params
 * @return {Promise<Response>} URL param query
 */
var queryBuilder = function queryBuilder(data) {
  if (data == null || _Object$keys(data).length === 0) return '';
  return _Object$keys(data).map(function (key) {
    return [key, data[key]].map(encodeURIComponent).join("=");
  }).join("&");
};

/**
 * Generate a public / private key pair from random mnemonic
 * @param {string} [passphrase] - Passphrase to get keys from
 * @return {Object} Private/public key pair
 */
var getKeys = function getKeys() {
  var passphrase = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

  // Get keys from passphrase if present or generate a random one
  var code = passphrase == null ? new Mnemonic() : passphrase;
  var keys = bcorejs.crypto.getKeys(code.toString());
  var address = bcorejs.crypto.getAddress(keys.publicKey);
  return {
    publicKey: keys.publicKey,
    privateKey: keys.d.toBuffer().toString('hex'),
    passphrase: code.toString(),
    address: address
  };
};

/**
 * Format network type
 * @param {string} network - Network type
 * @return {string} Formatted network type
 */
var formatNetworkType = function formatNetworkType(network) {
  return network.charAt(0).toUpperCase() + network.slice(1);
};

/**
 * Get the second signature fee
 * @return {Promise<Response>} Fee
 */
var getSecondSignatureFee = function getSecondSignatureFee() {
  return query('api/signatures/fee').then(function (res) {
    return res.fee;
  });
};

/**
 * Get public key from address
 * @param {string} - Address to get public key
 * @return {Promise<Response>} Public key
 */
var getPublicKey = function getPublicKey(address) {
  return query('api/accounts/getPublicKey', {
    address: address
  }).then(function (res) {
    return res.publicKey;
  });
};

/**
 * Get balance from address
 * @param {string} - Address to get balance
 * @return {Promise<Response>} Balance
 */
var getBalance = function getBalance(address) {
  return query('api/accounts/getBalance', {
    address: address
  }).then(function (res) {
    return res.balance / Math.pow(10, 8);
  });
};

/**
 * Get account details
 * @param {string} - Address to get details
 * @return {Promise<Response>} Account details
 */
var getAccount = function getAccount(address) {
  return query('api/accounts', {
    address: address
  }).then(function (res) {
    return res.account;
  });
};

/**
 * Get delegates list from address
 * @param {string} - Address to get vote delegates
 * @return {Promise<Response>} Array of delegates
 */
var getDelegatesFromAddress = function getDelegatesFromAddress(address) {
  return query('api/accounts/delegates', {
    address: address
  }).then(function (res) {
    return res.delegates;
  });
};

/**
 * Get the delegates register fee
 * @return {Promise<Response>} Delegates register fee
 */
var getDelegatesFee = function getDelegatesFee() {
  return query('api/accounts/delegates/fee').then(function (res) {
    return res.fee;
  });
};

/**
 * Return delegates list
 * @return {Promise<Response>} Array of delegates
 */
var getDelegates = function getDelegates(options) {
  var params = queryBuilder(options);
  return query('api/delegates?' + params).then(function (res) {
    return res.delegates;
  });
};

/**
 * Return delegates infos by username
 * @param {string} - Delegate username
 * @return {Promise<Response>} Delegate infos
 */
var getDelegateByUsername = function getDelegateByUsername(username) {
  return query('api/delegates/get', {
    username: username
  }).then(function (res) {
    return res.delegate;
  });
};

/**
 * Return delegates infos by public key
 * @param {string} - Delegate public key
 * @return {Promise<Response>} Delegate infos
 */
var getDelegateByPublicKey = function getDelegateByPublicKey(publicKey) {
  return query('api/delegates/get', {
    publicKey: publicKey
  }).then(function (res) {
    return res.delegate;
  });
};

/**
 * Return next forgers delegates list
 * @return {Promise<Response>} Delegates list
 */
var getNextForgers = function getNextForgers() {
  return query('api/delegates/getNextForgers').then(function (res) {
    return res.delegates;
  });
};

/**
 * Return voters list from delegate public key
 * @param {string} - Delegate public key
 * @return {Promise<Response>} Voters list
 */
var getDelegateVoters = function getDelegateVoters(publicKey) {
  return query('api/delegates/voters', {
    publicKey: publicKey
  }).then(function (res) {
    return res.accounts;
  });
};

/**
 * Return delegates matching query
 * @param {string} q - Search query
 * @param {Object} options - Search params
 * @return {Promise<Response>} Array of delegates
 */
var searchDelegate = function searchDelegate(q, options) {
  var params = queryBuilder(options);
  return query('api/delegates/search?q=' + q + '&' + params).then(function (res) {
    return res.delegates;
  });
};

/**
 * Get an array of the last transactions on the network
 * @return {Promise<Response>} Transactions array
 */
var getTransactions = function getTransactions(options) {
  var params = queryBuilder(options);
  return query('api/transactions?' + params).then(function (res) {
    return res.transactions;
  });
};

/**
 * Get transaction list
 * @param {string} - Address to get transactions from
 * @return {Promise<Response>} Transactions
 */
var getTransactionsFromAddress = function getTransactionsFromAddress(address, options) {
  return getTransactions(_extends({
    recipientId: address,
    senderId: address
  }, options)).then(function (res) {
    return res;
  });
};

/**
 * Get transaction by tx id
 * @param {string} - Transaction id
 * @return {Promise<Response>} Transaction details
 */
var getTransactionById = function getTransactionById(txid) {
  return query('api/transactions/get', {
    id: txid
  }).then(function (res) {
    return res.transaction;
  });
};

/**
 * Create a transaction
 * @param {Object} - Transaction parameters
 * @return {Object} Transaction
 */
var createTransaction = function createTransaction(data) {
  var amount = data.amount * Math.pow(10, 8);
  return bcorejs.transaction.createTransaction(data.to, amount, data.message, data.passphrase, data.secondPassphrase);
};

/**
 * Create a vote transaction
 * @param {Object} - Transaction parameters
 * @return {Object} Transaction
 */
var createVoteTransaction = function createVoteTransaction(data) {
  return bcorejs.vote.createVote(data.passphrase, data.delegates, data.secondPassphrase);
};

/**
 * Create a second signature transaction
 * @param {string} - Passphrase from account
 * @param {string} - secondPassphrase to set
 * @return {Object} Transaction
 */
var createSecondPassTransaction = function createSecondPassTransaction(passphrase, secondPassphrase) {
  return bcorejs.signature.createSignature(passphrase, secondPassphrase);
};

/**
 * Submit transaction to network
 * @param {Object} - Transaction details
 * @return {Promise<Response>} RPC response from sending transaction
 */
var sendTransaction = function sendTransaction(tx) {
  return post('peer/transactions', tx).then(function (res) {
    return res;
  });
};

/**
 * Get an array of unconfirmed transactions on the network
 * @return {Promise<Response>} Unconfirmed txs array
 */
var getUnconfirmedTransactions = function getUnconfirmedTransactions() {
  return query('api/transactions/unconfirmed').then(function (res) {
    return res.transactions;
  });
};

/**
 * Get peers list
 * @return {Promise<Response>} Peers array
 */
var getPeersList = function getPeersList() {
  return query('api/peers').then(function (res) {
    return res.peers;
  });
};

/**
 * Get peers list
 * @return {Promise<Response>} Peers array
 */
var getPeer = function getPeer(peerInfos) {
  return query('api/peers/get', {
    ip: peerInfos.ip,
    port: peerInfos.port
  }).then(function (res) {
    return res.peer;
  });
};

/**
 * Get peer version
 * @return {Promise<Response>} Peer version
 */
var getPeerVersion = function getPeerVersion() {
  return query('api/peers/version').then(function (res) {
    return res.version;
  });
};

exports.getPublicKey = getPublicKey;
exports.getBalance = getBalance;
exports.getAccount = getAccount;
exports.getDelegatesFromAddress = getDelegatesFromAddress;
exports.getDelegatesFee = getDelegatesFee;
exports.getDelegates = getDelegates;
exports.getDelegateByUsername = getDelegateByUsername;
exports.getDelegateByPublicKey = getDelegateByPublicKey;
exports.getNextForgers = getNextForgers;
exports.getDelegateVoters = getDelegateVoters;
exports.searchDelegate = searchDelegate;
exports.getTransactions = getTransactions;
exports.getTransactionsFromAddress = getTransactionsFromAddress;
exports.getTransactionById = getTransactionById;
exports.createTransaction = createTransaction;
exports.createVoteTransaction = createVoteTransaction;
exports.createSecondPassTransaction = createSecondPassTransaction;
exports.sendTransaction = sendTransaction;
exports.getUnconfirmedTransactions = getUnconfirmedTransactions;
exports.networksType = networksType;
exports.bcorejs = bcorejs;
exports.query = query;
exports.post = post;
exports.getEndpoint = getEndpoint;
exports.getNetwork = getNetwork;
exports.setNetwork = setNetwork;
exports.getSeeds = getSeeds;
exports.getAllSeeds = getAllSeeds;
exports.queryBuilder = queryBuilder;
exports.getKeys = getKeys;
exports.getSecondSignatureFee = getSecondSignatureFee;
exports.getBlocks = getBlocks;
exports.getBlock = getBlock;
exports.getBlocksHeight = getBlocksHeight;
exports.getBlockchainFee = getBlockchainFee;
exports.getNetHash = getNetHash;
exports.getEpoch = getEpoch;
exports.getReward = getReward;
exports.getSupply = getSupply;
exports.getStatus = getStatus;
exports.getPeersList = getPeersList;
exports.getPeer = getPeer;
exports.getPeerVersion = getPeerVersion;
//# sourceMappingURL=index.js.map
